import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import org.apache.log4j.Logger;
import org.drools.KnowledgeBase;
import org.drools.SystemEventListenerFactory;
import org.drools.builder.KnowledgeBuilder;
import org.drools.builder.KnowledgeBuilderFactory;
import org.drools.builder.ResourceType;
import org.drools.io.ResourceFactory;
import org.drools.runtime.StatefulKnowledgeSession;
import org.jbpm.process.workitem.wsht.LocalHTWorkItemHandler;
import org.jbpm.task.Content;
import org.jbpm.task.Task;
import org.jbpm.task.User;
import org.jbpm.task.query.TaskSummary;
import org.jbpm.task.service.TaskServiceSession;
import org.jbpm.task.service.local.LocalTaskService;
import org.jbpm.task.utils.OnErrorAction;

public class HelloJBPM5 {

	static Logger logger = Logger.getLogger(HelloJBPM5.class);

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		// START
		try {
			KnowledgeBase kbase = readKnowledgeBase();
			StatefulKnowledgeSession ksession = kbase
					.newStatefulKnowledgeSession();

			//
			EntityManagerFactory emf = Persistence.createEntityManagerFactory("org.jbpm.task");
			org.jbpm.task.service.TaskService taskService = new org.jbpm.task.service.TaskService(
					emf, SystemEventListenerFactory.getSystemEventListener());
			
			TaskServiceSession taskSession = taskService.createSession();
			taskSession.addUser(new User("Administrator"));
			taskSession.addUser(new User("nemo"));
			
			LocalTaskService localTaskService = new LocalTaskService(taskService);
			LocalHTWorkItemHandler humanTaskHandler = new LocalHTWorkItemHandler(localTaskService, ksession, OnErrorAction.RETHROW);
			humanTaskHandler.connect();
			ksession.getWorkItemManager().registerWorkItemHandler("Human Task",	humanTaskHandler);

			// ---------------------

			ksession.startProcess("HelloWorldWorldFlow", null);
			
			List<TaskSummary> taskSummaryList = localTaskService.getTasksAssignedAsPotentialOwner("nemo", "en-UK");
			for (TaskSummary taskSummary : taskSummaryList) {
				System.out.println(taskSummary.getId() + "\t" + taskSummary.getName() + "\t" + taskSummary.getStatus());
				Task task = localTaskService.getTask(taskSummary.getId());
				long documentContentId = task.getTaskData().getDocumentContentId();
				Content content = localTaskService.getContent(documentContentId);
				System.out.println(content.getId() + " => " + content.toString());
				if (content != null){
		            ByteArrayInputStream bais = new ByteArrayInputStream(content.getContent());
		            ObjectInputStream ois;

		            try {
		                ois = new ObjectInputStream(bais);
		                Object obj = ois.readObject();
		                logger.debug("Object = " + obj.getClass());
		            } catch (IOException e) {
		                e.printStackTrace();
		           } catch (ClassNotFoundException e) {
		                e.printStackTrace();

		            }
		        }
				localTaskService.start(taskSummary.getId(), "nemo");
				Map<String, Object> results = new HashMap<String, Object>();
				results.put("answer", "GOOD");
				localTaskService.completeWithResults(taskSummary.getId(), "nemo", results);	
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private static KnowledgeBase readKnowledgeBase() throws Exception {
		System.out.println("Loading knowledge base.");
		KnowledgeBuilder kbuilder = KnowledgeBuilderFactory
				.newKnowledgeBuilder();
		kbuilder.add(
				ResourceFactory.newClassPathResource("HelloWorldFlow.bpmn"),
				ResourceType.BPMN2);
		if (kbuilder.hasErrors()) {
			logger.error(kbuilder.getErrors().toString());
			throw new RuntimeException(
					"Unable to compile \"HelloWorldFlow.bpmn\".");
		}
		return kbuilder.newKnowledgeBase();
	}

}
